@@ -54,6 +54,7 @@ import org.w3c.dom.Attr;
 import org.w3c.dom.CDATASection;
 import org.w3c.dom.Comment;
 import org.w3c.dom.Document;
+import org.w3c.dom.DocumentFragment;
 import org.w3c.dom.Element;
 import org.w3c.dom.EntityReference;
 import org.w3c.dom.NamedNodeMap;
@@ -64,6 +65,7 @@ import org.w3c.dom.Text;
 import org.xml.sax.InputSource;
 import org.xml.sax.XMLReader;
 
+import org.apache.cxf.common.classloader.ClassLoaderUtils;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.util.StringUtils;
 import org.apache.cxf.helpers.DOMUtils;
@@ -95,6 +97,22 @@ public final class StaxUtils {
     private StaxUtils() {
     }
 
+    /**
+     * CXF works with multiple STaX parsers. When we can't find any other way to work 
+     * against the different parsers, this can be used to condition code. Note: if you've got
+     * Woodstox in the class path without being the default provider, this will return
+     * the wrong answer.
+     * @return true if Woodstox is in the classpath. 
+     */
+    public static boolean isWoodstox() {
+        try {
+            ClassLoaderUtils.loadClass("org.codehaus.stax2.XMLStreamReader2", StaxUtils.class);
+        } catch (ClassNotFoundException e) {
+            return false;
+        }
+        return true;
+    }
+    
     /**
      * Return a cached, namespace-aware, factory.
      * @return
@@ -129,7 +147,10 @@ public final class StaxUtils {
 
     public static XMLStreamWriter createXMLStreamWriter(Writer out) {
         try {
-            return getXMLOutputFactory().createXMLStreamWriter(out);
+            XMLOutputFactory factory = getXMLOutputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamWriter(out);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Cant' create XMLStreamWriter", e);
         }
@@ -145,7 +166,10 @@ public final class StaxUtils {
         }
 
         try {
-            return getXMLOutputFactory().createXMLStreamWriter(out, encoding);
+            XMLOutputFactory factory = getXMLOutputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamWriter(out, encoding);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Cant' create XMLStreamWriter", e);
         }
@@ -153,7 +177,10 @@ public final class StaxUtils {
     
     public static XMLStreamWriter createXMLStreamWriter(Result r) {
         try {
-            return getXMLOutputFactory().createXMLStreamWriter(r);
+            XMLOutputFactory factory = getXMLOutputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamWriter(r);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Cant' create XMLStreamWriter", e);
         }
@@ -161,7 +188,10 @@ public final class StaxUtils {
 
     public static XMLStreamReader createFilteredReader(XMLStreamReader reader, StreamFilter filter) {
         try {
-            return getXMLInputFactory().createFilteredReader(reader, filter);
+            XMLInputFactory factory = getXMLInputFactory();
+            synchronized (factory) {
+                return factory.createFilteredReader(reader, filter);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Cant' create XMLStreamReader", e);
         }
@@ -642,6 +672,15 @@ public final class StaxUtils {
         case Node.DOCUMENT_NODE:
             writeDocument((Document)n, writer, repairing);
             break;
+        case Node.DOCUMENT_FRAGMENT_NODE: {
+            DocumentFragment frag = (DocumentFragment)n;
+            Node child = frag.getFirstChild();
+            while (child != null) {
+                writeNode(child, writer, repairing);
+                child = child.getNextSibling();
+            }
+            break;
+        }
         default:
             throw new IllegalStateException("Found type: " + n.getClass().getName());
         }        
@@ -685,7 +724,7 @@ public final class StaxUtils {
             e.setPrefix(reader.getPrefix());
         }
 
-        parent.appendChild(e);
+        e = (Element)parent.appendChild(e);
 
         for (int ns = 0; ns < reader.getNamespaceCount(); ns++) {
             String uri = reader.getNamespaceURI(ns);
@@ -819,7 +858,10 @@ public final class StaxUtils {
         }
 
         try {
-            return getXMLInputFactory().createXMLStreamReader(in, encoding);
+            XMLInputFactory factory = getXMLInputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamReader(in, encoding);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Couldn't parse stream.", e);
         }
@@ -831,7 +873,20 @@ public final class StaxUtils {
      */
     public static XMLStreamReader createXMLStreamReader(InputStream in) {
         try {
-            return getXMLInputFactory().createXMLStreamReader(in);
+            XMLInputFactory factory = getXMLInputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamReader(in);
+            }
+        } catch (XMLStreamException e) {
+            throw new RuntimeException("Couldn't parse stream.", e);
+        }
+    }
+    public static XMLStreamReader createXMLStreamReader(String systemId, InputStream in) {
+        try {
+            XMLInputFactory factory = getXMLInputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamReader(systemId, in);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Couldn't parse stream.", e);
         }
@@ -859,8 +914,10 @@ public final class StaxUtils {
                     return new W3CDOMStreamReader(el);
                 }
             }
-            
-            return getXMLInputFactory().createXMLStreamReader(source);
+            XMLInputFactory factory = getXMLInputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamReader(source);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Couldn't parse stream.", e);
         }
@@ -873,7 +930,10 @@ public final class StaxUtils {
     public static XMLStreamReader createXMLStreamReader(Reader reader) {
 
         try {
-            return getXMLInputFactory().createXMLStreamReader(reader);
+            XMLInputFactory factory = getXMLInputFactory();
+            synchronized (factory) {
+                return factory.createXMLStreamReader(reader);
+            }
         } catch (XMLStreamException e) {
             throw new RuntimeException("Couldn't parse stream.", e);
         }
