@@ -108,7 +108,14 @@ public class LoggingOutInterceptor extends AbstractPhaseInterceptor {
         }
         
         public void onClose(CachedOutputStream cos) {
-            final LoggingMessage buffer = new LoggingMessage("Outbound Message\n---------------------------");
+            String id = (String)message.getExchange().get(LoggingMessage.ID_KEY);
+            if (id == null) {
+                id = LoggingMessage.nextId();
+                message.getExchange().put(LoggingMessage.ID_KEY, id);
+            }
+            final LoggingMessage buffer 
+                = new LoggingMessage("Outbound Message\n---------------------------",
+                                     id);
             
             String encoding = (String)message.get(Message.ENCODING);
 
@@ -116,8 +123,15 @@ public class LoggingOutInterceptor extends AbstractPhaseInterceptor {
                 buffer.getEncoding().append(encoding);
             }            
             
+            String address = (String)message.get(Message.ENDPOINT_ADDRESS);
+            if (address != null) {
+                buffer.getAddress().append(address);
+            }
+            String ct = (String)message.get(Message.CONTENT_TYPE);
+            if (ct != null) {
+                buffer.getContentType().append(ct);
+            }
             Object headers = message.get(Message.PROTOCOL_HEADERS);
-
             if (headers != null) {
                 buffer.getHeader().append(headers);
             }
