@@ -50,7 +50,6 @@ import javax.wsdl.extensions.ElementExtensible;
 import javax.wsdl.extensions.ExtensibilityElement;
 import javax.xml.namespace.QName;
 import javax.xml.parsers.ParserConfigurationException;
-import javax.xml.xpath.XPathConstants;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
@@ -62,7 +61,6 @@ import org.apache.cxf.NSManager;
 import org.apache.cxf.common.WSDLConstants;
 import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.helpers.XMLUtils;
-import org.apache.cxf.helpers.XPathUtils;
 import org.apache.cxf.service.model.AbstractMessageContainer;
 import org.apache.cxf.service.model.AbstractPropertiesHolder;
 import org.apache.cxf.service.model.BindingFaultInfo;
@@ -78,7 +76,6 @@ import org.apache.cxf.service.model.OperationInfo;
 import org.apache.cxf.service.model.SchemaInfo;
 import org.apache.cxf.service.model.ServiceInfo;
 import org.apache.cxf.wsdl.WSDLManager;
-import org.apache.ws.commons.schema.utils.NamespacePrefixList;
 
 /**
  * Consume a set of service definitions and produce a WSDL model. The ServiceInfo objects
@@ -245,39 +242,6 @@ public class ServiceWSDLBuilder {
         }
     }
     
-    /**
-     * For a schema, require all namespace with prefixes to be imported. In theory,
-     * if a namespace has a prefix but is not used, the import is not needed, and could
-     * cause a problem. In practice, the code generating these schemas should not be adding
-     * spurious prefixes. Hypothetically, we could check that, in the overall WSDL, that
-     * all of the schemas that we are importing are accounted for. However, that's a validation
-     * that is best left to the validator.
-     * @param schemaInfo Schema to process.
-     */
-    protected void addRequiredSchemaImports(SchemaInfo schemaInfo) {
-        Element schemaElement = schemaInfo.getElement();
-        // we need an import for every namespace except the main one.
-        String schemaNamespace = schemaInfo.getNamespaceURI();
-        Map<String, String> queryPrefixMap = new HashMap<String, String>();
-        queryPrefixMap.put("xs", WSDLConstants.NS_SCHEMA_XSD);
-        XPathUtils xpu = new XPathUtils(queryPrefixMap);
-        NamespacePrefixList schemaPrefixes = schemaInfo.getSchema().getNamespaceContext();
-        for (String prefix : schemaPrefixes.getDeclaredPrefixes()) {
-            String namespace = schemaPrefixes.getNamespaceURI(prefix);
-            if (!namespace.equals(schemaNamespace) 
-                && !namespace.equals(WSDLConstants.NS_SCHEMA_XSD)
-                && !xpu.isExist("xs:import[@namespace='" + namespace + "']", 
-                                 schemaElement, 
-                                 XPathConstants.NODE)) {
-                Element importElement = XMLUtils.createElementNS(schemaElement.getOwnerDocument(), 
-                                                                 WSDLConstants.NS_SCHEMA_XSD,
-                                                                 "import");
-                importElement.setAttribute("namespace", namespace);
-                schemaElement.insertBefore(importElement, schemaElement.getFirstChild());
-            }
-        }
-    }
-
     protected void buildTypes(final Collection<SchemaInfo> schemas,
                               final Map<String, SchemaInfo> imports,
                               final Definition def) {
@@ -314,8 +278,7 @@ public class ServiceWSDLBuilder {
                                
                 imports.put(name, schemaInfo);
             }
-            // add imports for those schemata which are inlined.
-            addRequiredSchemaImports(schemaInfo);
+
         }
         if (useSchemaImports) {
             SchemaImpl schemaImpl = new SchemaImpl();
@@ -526,6 +489,7 @@ public class ServiceWSDLBuilder {
                 if (operationInfo.isOneWay()) {
                     operation.setStyle(OperationType.ONE_WAY);
                 }
+                this.addExtensibilityElements(operation, getWSDL11Extensors(operationInfo));
                 Input input = def.createInput();
                 input.setName(operationInfo.getInputName());
                 Message message = def.createMessage();
@@ -533,6 +497,7 @@ public class ServiceWSDLBuilder {
                 this.addExtensibilityAttributes(input, getInputExtensionAttributes(operationInfo));
                 input.setMessage(message);
                 operation.setInput(input);
+                operation.setParameterOrdering(operationInfo.getParameterOrdering());
                 
                 if (operationInfo.getOutput() != null) {
                     Output output = def.createOutput();
