@@ -19,18 +19,16 @@
 
 package org.apache.cxf.jaxb;
 
-import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.io.OutputStream;
+import java.lang.reflect.Field;
+import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
-import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
@@ -40,42 +38,49 @@ import java.util.logging.Logger;
 import java.util.regex.Pattern;
 
 import javax.xml.bind.JAXBContext;
+import javax.xml.bind.JAXBElement;
 import javax.xml.bind.JAXBException;
 import javax.xml.bind.Marshaller;
-import javax.xml.bind.SchemaOutputResolver;
 import javax.xml.bind.Unmarshaller;
+import javax.xml.bind.ValidationEventHandler;
+import javax.xml.bind.annotation.XmlElement;
+import javax.xml.bind.annotation.XmlElementDecl;
+import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.stream.XMLEventReader;
 import javax.xml.stream.XMLEventWriter;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
-import javax.xml.transform.Result;
 import javax.xml.transform.dom.DOMResult;
 import javax.xml.transform.dom.DOMSource;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Node;
 
-import com.sun.xml.bind.marshaller.NamespacePrefixMapper;
-import com.sun.xml.bind.v2.ContextFactory;
-import com.sun.xml.bind.v2.runtime.JAXBContextImpl;
+import org.xml.sax.SAXException;
 
 import org.apache.cxf.common.i18n.Message;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.util.CacheMap;
+import org.apache.cxf.common.util.CachedClass;
 import org.apache.cxf.common.util.PackageUtils;
 import org.apache.cxf.common.util.StringUtils;
 import org.apache.cxf.common.xmlschema.SchemaCollection;
 import org.apache.cxf.databinding.AbstractDataBinding;
+import org.apache.cxf.databinding.AbstractWrapperHelper;
 import org.apache.cxf.databinding.DataReader;
 import org.apache.cxf.databinding.DataWriter;
+import org.apache.cxf.databinding.WrapperCapableDatabinding;
+import org.apache.cxf.databinding.WrapperHelper;
+import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.jaxb.io.DataReaderImpl;
 import org.apache.cxf.jaxb.io.DataWriterImpl;
+import org.apache.cxf.resource.URIResolver;
 import org.apache.cxf.service.Service;
 import org.apache.cxf.service.factory.ServiceConstructionException;
 import org.apache.cxf.service.model.ServiceInfo;
 import org.apache.cxf.ws.addressing.ObjectFactory;
 
-public final class JAXBDataBinding extends AbstractDataBinding {
+public class JAXBDataBinding extends AbstractDataBinding  implements WrapperCapableDatabinding {
     public static final String SCHEMA_RESOURCE = "SCHEMRESOURCE";
     public static final String MTOM_THRESHOLD = "org.apache.cxf.jaxb.mtomThreshold";
 
@@ -115,27 +120,85 @@ public final class JAXBDataBinding extends AbstractDataBinding {
 
     }
     
-    private static final class CachedClassOrNull {
-        private Class<?> cachedClass;
-
-        public CachedClassOrNull(Class<?> cachedClass) {
-            this.cachedClass = cachedClass;
-        }
-
-        public Class<?> getCachedClass() {
-            return cachedClass;
-        }
-
-        public void setCachedClass(Class<?> cachedClass) {
-            this.cachedClass = cachedClass;
-        }
-    }
-
     private static final Map<Set<Class<?>>, CachedContextAndSchemas> JAXBCONTEXT_CACHE 
         = new CacheMap<Set<Class<?>>, CachedContextAndSchemas>();
     
-    private static final Map<String, CachedClassOrNull> OBJECT_FACTORY_CACHE
-        = new CacheMap<String, CachedClassOrNull>();
+    private static final Map<Package, CachedClass> OBJECT_FACTORY_CACHE
+        = new CacheMap<Package, CachedClass>();
+    
+    private static final Map<String, DOMResult> BUILT_IN_SCHEMAS = new HashMap<String, DOMResult>();
+    static {
+        URIResolver resolver = new URIResolver();
+        try {
+            resolver.resolve("", "classpath:/schemas/wsdl/ws-addr-wsdl.xsd", JAXBDataBinding.class);
+            if (resolver.isResolved()) {
+                InputStream ins = resolver.getInputStream();
+                Document doc = XMLUtils.parse(ins);
+                ins.close();
+                DOMResult dr = new DOMResult(doc, "classpath:/schemas/wsdl/ws-addr-wsdl.xsd");
+                BUILT_IN_SCHEMAS.put("http://www.w3.org/2005/02/addressing/wsdl", dr);
+                resolver.unresolve();
+            }
+        } catch (IOException e) {
+            //IGNORE
+        } catch (ParserConfigurationException e) {
+            //IGNORE
+        } catch (SAXException e) {
+            //IGNORE
+        }
+        try {
+            resolver.resolve("", "classpath:/schemas/wsdl/ws-addr.xsd", JAXBDataBinding.class);
+            if (resolver.isResolved()) {
+                InputStream ins = resolver.getInputStream();
+                Document doc = XMLUtils.parse(ins);
+                ins.close();
+                DOMResult dr = new DOMResult(doc, "classpath:/schemas/wsdl/ws-addr.xsd");
+                BUILT_IN_SCHEMAS.put("http://www.w3.org/2005/08/addressing", dr);
+                resolver.unresolve();
+            }
+        } catch (IOException e) {
+            //IGNORE
+        } catch (ParserConfigurationException e) {
+            //IGNORE
+        } catch (SAXException e) {
+            //IGNORE
+        }
+        try {
+            resolver.resolve("", "classpath:/schemas/wsdl/wsrm.xsd", JAXBDataBinding.class);
+            if (resolver.isResolved()) {
+                InputStream ins = resolver.getInputStream();
+                Document doc = XMLUtils.parse(ins);
+                ins.close();
+                DOMResult dr = new DOMResult(doc, "classpath:/schemas/wsdl/wsrm.xsd");
+                BUILT_IN_SCHEMAS.put("http://schemas.xmlsoap.org/ws/2005/02/rm", dr);
+                resolver.unresolve();
+            }
+        } catch (IOException e) {
+            //IGNORE
+        } catch (ParserConfigurationException e) {
+            //IGNORE
+        } catch (SAXException e) {
+            //IGNORE
+        }
+        try {
+            resolver.resolve("", "classpath:/schemas/wsdl/wsrm.xsd", JAXBDataBinding.class);
+            if (resolver.isResolved()) {
+                InputStream ins = resolver.getInputStream();
+                Document doc = XMLUtils.parse(ins);
+                ins.close();
+                DOMResult dr = new DOMResult(doc, "classpath:/schemas/wsdl/wsrm.xsd");
+                BUILT_IN_SCHEMAS.put("http://schemas.xmlsoap.org/ws/2005/02/rm", dr);
+                resolver.unresolve();
+            }
+        } catch (IOException e) {
+            //IGNORE
+        } catch (ParserConfigurationException e) {
+            //IGNORE
+        } catch (SAXException e) {
+            //IGNORE
+        }
+    }
+    
 
     Class[] extraClass;
 
@@ -149,7 +212,10 @@ public final class JAXBDataBinding extends AbstractDataBinding {
     private Map<String, Object> unmarshallerProperties = Collections.emptyMap();
     private Unmarshaller.Listener unmarshallerListener;
     private Marshaller.Listener marshallerListener;
+    private ValidationEventHandler validationEventHandler;
     
+    private boolean unwrapJAXBElement = true;
+
     private boolean qualifiedSchemas;
     private Service service;
 
@@ -163,7 +229,7 @@ public final class JAXBDataBinding extends AbstractDataBinding {
     public JAXBDataBinding(Class<?>... classes) throws JAXBException {
         contextClasses = new LinkedHashSet<Class<?>>();
         contextClasses.addAll(Arrays.asList(classes));
-        setContext(createJAXBContext(contextClasses));
+        setContext(createJAXBContext(contextClasses)); //NOPMD - specifically allow this
     }
 
     public JAXBDataBinding(JAXBContext context) {
@@ -175,31 +241,10 @@ public final class JAXBDataBinding extends AbstractDataBinding {
         return context;
     }
 
-    public void setContext(JAXBContext ctx) {
+    public final void setContext(JAXBContext ctx) {
         context = ctx;
     }
 
-    public NamespacePrefixMapper getNamespacePrefixMapper() {
-        Map<String, String> mappings = getDeclaredNamespaceMappings();
-        if (mappings == null) {
-            mappings = Collections.emptyMap();
-        }
-
-        final Map<String, String> closedMappings = mappings;
-
-        NamespacePrefixMapper mapper = new NamespacePrefixMapper() {
-            @Override
-            public String getPreferredPrefix(String namespaceUri, String suggestion, boolean requirePrefix) {
-                String prefix = closedMappings.get(namespaceUri);
-                if (prefix != null) {
-                    return prefix;
-                }
-                return suggestion;
-            }
-        };
-        return mapper;
-    }
-
     @SuppressWarnings("unchecked")
     public <T> DataWriter<T> createWriter(Class<T> c) {
 
@@ -233,11 +278,11 @@ public final class JAXBDataBinding extends AbstractDataBinding {
     public <T> DataReader<T> createReader(Class<T> c) {
         DataReader<T> dr = null;
         if (c == XMLStreamReader.class) {
-            dr = (DataReader<T>)new DataReaderImpl<XMLStreamReader>(this);
+            dr = (DataReader<T>)new DataReaderImpl<XMLStreamReader>(this, unwrapJAXBElement);
         } else if (c == XMLEventReader.class) {
-            dr = (DataReader<T>)new DataReaderImpl<XMLEventReader>(this);
+            dr = (DataReader<T>)new DataReaderImpl<XMLEventReader>(this, unwrapJAXBElement);
         } else if (c == Node.class) {
-            dr = (DataReader<T>)new DataReaderImpl<Node>(this);
+            dr = (DataReader<T>)new DataReaderImpl<Node>(this, unwrapJAXBElement);
         }
 
         return dr;
@@ -255,7 +300,6 @@ public final class JAXBDataBinding extends AbstractDataBinding {
             return;
         }
 
-        CachedContextAndSchemas cachedContextAndSchemas = null;
 
         contextClasses = new LinkedHashSet<Class<?>>();
         for (ServiceInfo serviceInfo : service.getServiceInfos()) {
@@ -269,35 +313,37 @@ public final class JAXBDataBinding extends AbstractDataBinding {
         }
 
         String tns = service.getName().getNamespaceURI();
+        CachedContextAndSchemas cachedContextAndSchemas = null;
         JAXBContext ctx = null;
         try {
             if (service.getServiceInfos().size() > 0) {
                 tns = service.getServiceInfos().get(0).getInterface().getName().getNamespaceURI();
             }
-            ctx = createJAXBContext(contextClasses, tns);
-            cachedContextAndSchemas = JAXBCONTEXT_CACHE.get(contextClasses);
+            cachedContextAndSchemas = createJAXBContextAndSchemas(contextClasses, tns);
         } catch (JAXBException e1) {
             // load jaxb needed class and try to create jaxb context for more
             // times
             boolean added = addJaxbObjectFactory(e1);
-            while (ctx == null && added) {
+            while (cachedContextAndSchemas == null && added) {
                 try {
-                    synchronized (JAXBCONTEXT_CACHE) {
-                        ctx = JAXBContext.newInstance(contextClasses
-                            .toArray(new Class[contextClasses.size()]), null);
-                        cachedContextAndSchemas = new CachedContextAndSchemas(ctx);
-                        JAXBCONTEXT_CACHE.put(contextClasses, cachedContextAndSchemas);
-                    }
+                    ctx = JAXBContext.newInstance(contextClasses
+                                                  .toArray(new Class[contextClasses.size()]), null);
+                    cachedContextAndSchemas = new CachedContextAndSchemas(ctx);
                 } catch (JAXBException e) {
                     e1 = e;
                     added = addJaxbObjectFactory(e1);
                 }
             }
+
             if (ctx == null) {
                 throw new ServiceConstructionException(e1);
+            } else {
+                synchronized (JAXBCONTEXT_CACHE) {
+                    JAXBCONTEXT_CACHE.put(contextClasses, cachedContextAndSchemas);
+                }                
             }
         }
-
+        ctx = cachedContextAndSchemas.getContext();
         if (LOG.isLoggable(Level.FINE)) {
             LOG.log(Level.FINE, "CREATED_JAXB_CONTEXT", new Object[] {ctx, contextClasses});
         }
@@ -321,31 +367,46 @@ public final class JAXBDataBinding extends AbstractDataBinding {
             } else {
                 schemasFromCache = true;
             }
+            Set<DOMSource> bi = new LinkedHashSet<DOMSource>();
             if (schemas == null) {
-                schemas = new HashSet<DOMSource>();
+                schemas = new LinkedHashSet<DOMSource>();
                 try {
                     for (DOMResult r : generateJaxbSchemas()) {
-                        schemas.add(new DOMSource(r.getNode()));
+                        DOMSource src = new DOMSource(r.getNode(), r.getSystemId());
+                        if (BUILT_IN_SCHEMAS.containsValue(r)) {
+                            bi.add(src);
+                        } else {
+                            schemas.add(src);                            
+                        }
                     }
+                    //put any builtins at the end.   Anything that DOES import them 
+                    //will cause it to load automatically and we'll skip them later
+                    schemas.addAll(bi);
                 } catch (IOException e) {
                     throw new ServiceConstructionException(new Message("SCHEMA_GEN_EXC", LOG), e);
                 }
             }
             for (DOMSource r : schemas) {
+                if (bi.contains(r)) {
+                    String ns = ((Document)r.getNode()).getDocumentElement().getAttribute("targetNamespace");
+                    if (serviceInfo.getSchema(ns) != null) {
+                        continue;
+                    }
+                }
                 addSchemaDocument(serviceInfo, 
                                   col, 
                                  (Document)r.getNode(),
                                   r.getSystemId());
             }
 
-            JAXBContextImpl riContext;
-            if (context instanceof JAXBContextImpl) {
-                riContext = (JAXBContextImpl)context;
+            JAXBContext riContext;
+            if (context.getClass().getName().contains("com.sun.xml.")) {
+                riContext = context;
             } else {
                 // fall back if we're using another jaxb implementation
                 try {
-                    riContext = (JAXBContextImpl)ContextFactory.createContext(contextClasses
-                        .toArray(new Class[contextClasses.size()]), null);
+                    riContext = JAXBUtils.createRIContext(contextClasses
+                        .toArray(new Class[contextClasses.size()]), tns);
                 } catch (JAXBException e) {
                     throw new ServiceConstructionException(e);
                 }
@@ -370,33 +431,7 @@ public final class JAXBDataBinding extends AbstractDataBinding {
 
     // default access for tests.
     List<DOMResult> generateJaxbSchemas() throws IOException {
-        final List<DOMResult> results = new ArrayList<DOMResult>();
-
-        context.generateSchema(new SchemaOutputResolver() {
-            private Map<String, String> builtIns = new HashMap<String, String>();
-            {
-                builtIns.put("http://www.w3.org/2005/02/addressing/wsdl",
-                             "classpath:/schemas/wsdl/ws-addr-wsdl.xsd");
-                builtIns.put("http://www.w3.org/2005/08/addressing", "classpath:/schemas/wsdl/ws-addr.xsd");
-                builtIns.put("http://schemas.xmlsoap.org/ws/2005/02/rm", "classpath:/schemas/wsdl/wsrm.xsd");
-                builtIns.put("http://www.w3.org/2005/05/xmlmime", "classpath:/schemas/wsdl/ws-addr.xsd");
-            }
-
-            @Override
-            public Result createOutput(String ns, String file) throws IOException {
-                DOMResult result = new DOMResult();
-
-                if (builtIns.containsKey(ns)) {
-                    result.setSystemId(builtIns.get(ns));
-                    return result;
-                }
-                result.setSystemId(file);
-                results.add(result);
-                return result;
-            }
-        });
-
-        return results;
+        return JAXBUtils.generateJaxbSchemas(context, BUILT_IN_SCHEMAS);
     }
 
     public JAXBContext createJAXBContext(Set<Class<?>> classes) throws JAXBException {
@@ -404,16 +439,13 @@ public final class JAXBDataBinding extends AbstractDataBinding {
     }
 
     public JAXBContext createJAXBContext(Set<Class<?>> classes, String defaultNs) throws JAXBException {
-        Iterator it = classes.iterator();
-        String className = "";
-        Object remoteExceptionObject = null;
-        while (it.hasNext()) {
-            remoteExceptionObject = (Class)it.next();
-            className = remoteExceptionObject.toString();
-            if (!("".equals(className)) && className.contains("RemoteException")) {
-                it.remove();
-            }
-        }
+        return createJAXBContextAndSchemas(classes, defaultNs).getContext();
+    }
+    
+    public CachedContextAndSchemas createJAXBContextAndSchemas(Set<Class<?>> classes,
+                                                               String defaultNs) 
+        throws JAXBException {
+        
         // add user extra class into jaxb context
         if (extraClass != null && extraClass.length > 0) {
             for (Class clz : extraClass) {
@@ -421,85 +453,12 @@ public final class JAXBDataBinding extends AbstractDataBinding {
             }
         }
 
-        // try and read any jaxb.index files that are with the other classes.
-        // This should
-        // allow loading of extra classes (such as subclasses for inheritance
-        // reasons)
-        // that are in the same package. Also check for ObjectFactory classes
-        Map<String, InputStream> packages = new HashMap<String, InputStream>();
-        Map<String, ClassLoader> packageLoaders = new HashMap<String, ClassLoader>();
-        Set<Class<?>> objectFactories = new HashSet<Class<?>>();
-        for (Class<?> jcls : classes) {
-            String pkgName = PackageUtils.getPackageName(jcls);
-            if (!packages.containsKey(pkgName)) {
-                packages.put(pkgName, jcls.getResourceAsStream("jaxb.index"));
-                packageLoaders.put(pkgName, jcls.getClassLoader());
-                String objectFactoryClassName = pkgName + "." + "ObjectFactory";
-                Class<?> ofactory = null;
-                CachedClassOrNull cachedFactory = null;
-                synchronized (OBJECT_FACTORY_CACHE) {
-                    cachedFactory = OBJECT_FACTORY_CACHE.get(objectFactoryClassName);
-                }
-                if (cachedFactory != null) {
-                    ofactory = cachedFactory.getCachedClass();
-                }
-                if (ofactory == null) {
-                    try {
-                        ofactory = Class.forName(objectFactoryClassName, false, jcls
-                                                 .getClassLoader());
-                        objectFactories.add(ofactory);
-                        addToObjectFactoryCache(objectFactoryClassName, ofactory);
-                    } catch (ClassNotFoundException e) {
-                        addToObjectFactoryCache(objectFactoryClassName, null);
-                    }
-                } else {
-                    objectFactories.add(ofactory);                    
-                }
-            }
-        }
-        for (Map.Entry<String, InputStream> entry : packages.entrySet()) {
-            if (entry.getValue() != null) {
-                try {
-                    BufferedReader reader = new BufferedReader(new InputStreamReader(entry.getValue(),
-                                                                                     "UTF-8"));
-                    String pkg = entry.getKey();
-                    ClassLoader loader = packageLoaders.get(pkg);
-                    if (!StringUtils.isEmpty(pkg)) {
-                        pkg += ".";
-                    }
-
-                    String line = reader.readLine();
-                    while (line != null) {
-                        line = line.trim();
-                        if (line.indexOf("#") != -1) {
-                            line = line.substring(0, line.indexOf("#"));
-                        }
-                        if (!StringUtils.isEmpty(line)) {
-                            try {
-                                Class<?> ncls = Class.forName(pkg + line, false, loader);
-                                classes.add(ncls);
-                            } catch (Exception e) {
-                                // ignore
-                            }
-                        }
-                        line = reader.readLine();
-                    }
-                } catch (Exception e) {
-                    // ignore
-                } finally {
-                    try {
-                        entry.getValue().close();
-                    } catch (Exception e) {
-                        // ignore
-                    }
-                }
-            }
-        }
-        classes.addAll(objectFactories);
+        JAXBUtils.scanPackages(classes, OBJECT_FACTORY_CACHE);
         addWsAddressingTypes(classes);
 
         for (Class<?> clz : classes) {
-            if (clz.getName().endsWith("ObjectFactory")) {
+            if (clz.getName().endsWith("ObjectFactory")
+                && checkObjectFactoryNamespaces(clz)) {
                 // kind of a hack, but ObjectFactories may be created with empty
                 // namespaces
                 defaultNs = null;
@@ -518,22 +477,42 @@ public final class JAXBDataBinding extends AbstractDataBinding {
 
         CachedContextAndSchemas cachedContextAndSchemas = null;
         synchronized (JAXBCONTEXT_CACHE) {
-            if (!JAXBCONTEXT_CACHE.containsKey(classes)) {
-                JAXBContext ctx = JAXBContext.newInstance(classes.toArray(new Class[classes.size()]), map);
-                cachedContextAndSchemas = new CachedContextAndSchemas(ctx);
-                JAXBCONTEXT_CACHE.put(classes, cachedContextAndSchemas);
-            }
             cachedContextAndSchemas = JAXBCONTEXT_CACHE.get(classes);
         }
-
-        return cachedContextAndSchemas.getContext();
-    }
-
-    private void addToObjectFactoryCache(String objectFactoryClassName, Class<?> ofactory) {
-        synchronized (OBJECT_FACTORY_CACHE) {
-            OBJECT_FACTORY_CACHE.put(objectFactoryClassName, 
-                                     new CachedClassOrNull(ofactory));
+        if (cachedContextAndSchemas == null) {
+            JAXBContext ctx;
+            try {
+                ctx = JAXBContext.newInstance(classes.toArray(new Class[classes.size()]), map);
+            } catch (JAXBException ex) {
+                if (map.containsKey("com.sun.xml.bind.defaultNamespaceRemap")
+                    && ex.getMessage().contains("com.sun.xml.bind.defaultNamespaceRemap")) {
+                    map.put("com.sun.xml.internal.bind.defaultNamespaceRemap",
+                            map.remove("com.sun.xml.bind.defaultNamespaceRemap"));
+                    ctx = JAXBContext.newInstance(classes.toArray(new Class[classes.size()]), map);
+                } else {
+                    throw ex;
+                }
+            }
+            cachedContextAndSchemas = new CachedContextAndSchemas(ctx);
+            synchronized (JAXBCONTEXT_CACHE) {
+                JAXBCONTEXT_CACHE.put(classes, cachedContextAndSchemas);
+            }
         }
+
+        return cachedContextAndSchemas;
+    }
+    
+    private boolean checkObjectFactoryNamespaces(Class<?> clz) {
+        for (Method meth : clz.getMethods()) {
+            XmlElementDecl decl = meth.getAnnotation(XmlElementDecl.class);
+            if (decl != null 
+                && XmlElementDecl.GLOBAL.class.equals(decl.scope())
+                && StringUtils.isEmpty(decl.namespace())) {
+                return true;
+            }
+        }
+
+        return false;
     }
 
     private void addWsAddressingTypes(Set<Class<?>> classes) {
@@ -682,8 +661,25 @@ public final class JAXBDataBinding extends AbstractDataBinding {
     public void setMarshallerListener(Marshaller.Listener marshallerListener) {
         this.marshallerListener = marshallerListener;
     }
+    
+    
+    public ValidationEventHandler getValidationEventHandler() {
+        return validationEventHandler;
+    }
+
+    public void setValidationEventHandler(ValidationEventHandler validationEventHandler) {
+        this.validationEventHandler = validationEventHandler;
+    }
 
     
+    public boolean isUnwrapJAXBElement() {
+        return unwrapJAXBElement;
+    }
+
+    public void setUnwrapJAXBElement(boolean unwrapJAXBElement) {
+        this.unwrapJAXBElement = unwrapJAXBElement;
+    }
+
     public static void clearCaches() {
         synchronized (JAXBCONTEXT_CACHE) {
             JAXBCONTEXT_CACHE.clear();
@@ -693,4 +689,195 @@ public final class JAXBDataBinding extends AbstractDataBinding {
         }
     }
 
+    public WrapperHelper createWrapperHelper(Class<?> wrapperType, List<String> partNames,
+                                             List<String> elTypeNames, List<Class<?>> partClasses) {
+        List<Method> getMethods = new ArrayList<Method>(partNames.size());
+        List<Method> setMethods = new ArrayList<Method>(partNames.size());
+        List<Method> jaxbMethods = new ArrayList<Method>(partNames.size());
+        List<Field> fields = new ArrayList<Field>(partNames.size());
+        
+        Method allMethods[] = wrapperType.getMethods();
+        String packageName = PackageUtils.getPackageName(wrapperType);
+        
+        //if wrappertype class is generated by ASM,getPackage() always return null
+        if (wrapperType.getPackage() != null) {
+            packageName = wrapperType.getPackage().getName();
+        }
+        
+        String objectFactoryClassName = packageName + ".ObjectFactory";
+
+        Object objectFactory = null;
+        try {
+            objectFactory = wrapperType.getClassLoader().loadClass(objectFactoryClassName).newInstance();
+        } catch (Exception e) {
+            //ignore, probably won't need it
+        }
+        Method allOFMethods[];
+        if (objectFactory != null) {
+            allOFMethods = objectFactory.getClass().getMethods(); 
+        } else {
+            allOFMethods = new Method[0];
+        }
+        
+        for (int x = 0; x < partNames.size(); x++) {
+            String partName = partNames.get(x);            
+            if (partName == null) {
+                getMethods.add(null);
+                setMethods.add(null);
+                fields.add(null);
+                jaxbMethods.add(null);
+                continue;
+            }
+            
+            String elementType = elTypeNames.get(x);
+            
+            String getAccessor = JAXBUtils.nameToIdentifier(partName, JAXBUtils.IdentifierType.GETTER);
+            String setAccessor = JAXBUtils.nameToIdentifier(partName, JAXBUtils.IdentifierType.SETTER);
+            Method getMethod = null;
+            Method setMethod = null;
+            Class<?> valueClass = wrapperType;
+            
+            try {               
+                getMethod = valueClass.getMethod(getAccessor, AbstractWrapperHelper.NO_CLASSES); 
+            } catch (NoSuchMethodException ex) {
+                //ignore for now
+            }
+            
+            Field elField = getElField(partName, valueClass);
+            if (getMethod == null
+                && elementType != null
+                && "boolean".equals(elementType.toLowerCase())
+                && (elField == null
+                    || (!Collection.class.isAssignableFrom(elField.getType())
+                    && !elField.getType().isArray()))) {
+        
+                try {
+                    String newAcc = getAccessor.replaceFirst("get", "is");
+                    getMethod = wrapperType.getMethod(newAcc, AbstractWrapperHelper.NO_CLASSES); 
+                } catch (NoSuchMethodException ex) {
+                    //ignore for now
+                }            
+            }                        
+            if (getMethod == null 
+                && "return".equals(partName)) {
+                //RI generated code uses this
+                try {
+                    getMethod = valueClass.getMethod("get_return", AbstractWrapperHelper.NO_CLASSES);
+                } catch (NoSuchMethodException ex) {
+                    try {
+                        getMethod = valueClass.getMethod("is_return",
+                                                          new Class[0]);
+                    } catch (NoSuchMethodException ex2) {
+                        //ignore for now
+                    } 
+                }                
+            }
+            if (getMethod == null && elField != null) {
+                getAccessor = JAXBUtils.nameToIdentifier(elField.getName(), JAXBUtils.IdentifierType.GETTER);
+                setAccessor = JAXBUtils.nameToIdentifier(elField.getName(), JAXBUtils.IdentifierType.SETTER);
+                try {               
+                    getMethod = valueClass.getMethod(getAccessor, AbstractWrapperHelper.NO_CLASSES); 
+                } catch (NoSuchMethodException ex) {
+                    //ignore for now
+                }
+            }
+            String setAccessor2 = setAccessor;
+            if ("return".equals(partName)) {
+                //some versions of jaxb map "return" to "set_return" instead of "setReturn"
+                setAccessor2 = "set_return";
+            }
+
+            for (Method method : allMethods) {
+                if (method.getParameterTypes() != null && method.getParameterTypes().length == 1
+                    && (setAccessor.equals(method.getName())
+                        || setAccessor2.equals(method.getName()))) {
+                    setMethod = method;
+                    break;
+                }
+            }
+            
+            getMethods.add(getMethod);
+            setMethods.add(setMethod);
+            if (setMethod != null
+                && JAXBElement.class.isAssignableFrom(setMethod.getParameterTypes()[0])) {
+                
+                String methodName = "create" + wrapperType.getSimpleName()
+                    + setMethod.getName().substring(3);
+
+                for (Method m : allOFMethods) {
+                    if (m.getName().equals(methodName)) {
+                        jaxbMethods.add(m);
+                    }
+                }
+            } else {
+                jaxbMethods.add(null);
+            }
+            
+            if (elField != null) {
+                // JAXB Type get XmlElement Annotation
+                XmlElement el = elField.getAnnotation(XmlElement.class);
+                if (el != null
+                    && partName.equals(el.name())) {
+                    elField.setAccessible(true);
+                    fields.add(elField);
+                } else {
+                    fields.add(null);
+                } 
+            } else {
+                fields.add(null);
+            }
+            
+        }
+        
+        return createWrapperHelper(wrapperType,
+                                 setMethods.toArray(new Method[setMethods.size()]),
+                                 getMethods.toArray(new Method[getMethods.size()]),
+                                 jaxbMethods.toArray(new Method[jaxbMethods.size()]),
+                                 fields.toArray(new Field[fields.size()]),
+                                 objectFactory);
+    }
+    
+    private static Field getElField(String partName, Class<?> wrapperType) {
+        String fieldName = JAXBUtils.nameToIdentifier(partName, JAXBUtils.IdentifierType.VARIABLE);
+        for (Field field : wrapperType.getDeclaredFields()) {
+            XmlElement el = field.getAnnotation(XmlElement.class);
+            if (el != null
+                && partName.equals(el.name())) {
+                return field;
+            }
+            if (field.getName().equals(fieldName)) {
+                return field;
+            }
+        }        
+        return null;
+    }
+
+    
+    private static WrapperHelper createWrapperHelper(Class<?> wrapperType, Method setMethods[],
+                                                     Method getMethods[], Method jaxbMethods[],
+                                                     Field fields[], Object objectFactory) {
+
+        WrapperHelper wh = compileWrapperHelper(wrapperType, setMethods, getMethods, jaxbMethods, fields,
+                                                objectFactory);
+
+        if (wh == null) {
+            wh = new JAXBWrapperHelper(wrapperType, setMethods, getMethods, jaxbMethods, fields,
+                                       objectFactory);
+        }
+        return wh;
+    }
+
+    private static WrapperHelper compileWrapperHelper(Class<?> wrapperType, Method setMethods[],
+                                                      Method getMethods[], Method jaxbMethods[],
+                                                      Field fields[], Object objectFactory) {
+        try {
+            Class.forName("org.objectweb.asm.ClassWriter");
+            return WrapperHelperCompiler.compileWrapperHelper(wrapperType, setMethods, getMethods,
+                                                              jaxbMethods, fields, objectFactory);
+        } catch (ClassNotFoundException e) {
+            // ASM not found, just use reflection based stuff
+        }
+        return null;
+    }   
+
 }
