@@ -26,6 +26,7 @@ import java.lang.reflect.Method;
 import java.util.logging.Logger;
 
 import javax.jbi.messaging.DeliveryChannel;
+import javax.jbi.messaging.ExchangeStatus;
 import javax.jbi.messaging.InOut;
 import javax.jbi.messaging.MessageExchange;
 import javax.jbi.messaging.MessageExchangeFactory;
@@ -35,6 +36,7 @@ import javax.xml.namespace.QName;
 import javax.xml.transform.Source;
 import javax.xml.transform.stream.StreamSource;
 
+import org.apache.cxf.Bus;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.io.CachedOutputStream;
 import org.apache.cxf.message.Exchange;
@@ -91,7 +93,8 @@ public class JBIConduitOutputStream extends CachedOutputStream {
             QName interfaceName = new QName(ws.targetNamespace(), ws.name());
             QName serviceName = null;
             if (target != null) {
-                serviceName = EndpointReferenceUtils.getServiceName(target);
+                serviceName = EndpointReferenceUtils.getServiceName(target,
+                                                                    message.getExchange().get(Bus.class));
             } else {
                 serviceName = message.getExchange().get(org.apache.cxf.service.Service.class).getName();
             }
@@ -142,12 +145,13 @@ public class JBIConduitOutputStream extends CachedOutputStream {
                             "UNABLE.RETRIEVE.MESSAGE", LOG).toString());
                     }
                     inMessage.setContent(InputStream.class, ins);
+                    inMessage.put(MessageExchange.class, xchng);
                     
                     
                     conduit.getMessageObserver().onMessage(inMessage);
-                    
-                    
-                    
+
+                    xchng.setStatus(ExchangeStatus.DONE);
+                    channel.send(xchng);                    
                 } else {
                     channel.send(xchng);
                 }
@@ -161,8 +165,7 @@ public class JBIConduitOutputStream extends CachedOutputStream {
         } catch (IOException e) {
             throw e;
         } catch (Exception e) {
-            e.printStackTrace();
-            new IOException(e.toString());
+            throw new RuntimeException(e.toString());
         }
     }
 
