@@ -24,6 +24,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
+import java.lang.ref.WeakReference;
 import java.util.Map;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
@@ -36,14 +37,18 @@ import javax.jms.JMSException;
 import javax.jms.MessageListener;
 import javax.jms.Session;
 
+import org.apache.cxf.Bus;
+import org.apache.cxf.buslifecycle.BusLifeCycleListener;
+import org.apache.cxf.buslifecycle.BusLifeCycleManager;
 import org.apache.cxf.common.logging.LogUtils;
+import org.apache.cxf.configuration.ConfigurationException;
 import org.apache.cxf.message.Exchange;
 import org.apache.cxf.message.Message;
 import org.apache.cxf.message.MessageImpl;
+import org.apache.cxf.message.MessageUtils;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.transport.AbstractConduit;
 import org.apache.cxf.ws.addressing.EndpointReferenceType;
-import org.springframework.jms.connection.SingleConnectionFactory;
 import org.springframework.jms.core.JmsTemplate;
 import org.springframework.jms.core.MessageCreator;
 import org.springframework.jms.listener.DefaultMessageListenerContainer;
@@ -56,30 +61,27 @@ import org.springframework.jms.support.JmsUtils;
  * Message. This is then provided in the Exchange and also sent to the incomingObserver
  */
 public class JMSConduit extends AbstractConduit implements JMSExchangeSender, MessageListener {
+
     static final Logger LOG = LogUtils.getL7dLogger(JMSConduit.class);
     
     private static final String CORRELATED = JMSConduit.class.getName() + ".correlated";
-    private JMSListenerPool listenerPool;    
+    
     private EndpointInfo endpointInfo;
     private JMSConfiguration jmsConfig;
     private Map<String, Exchange> correlationMap;
-    private Map<String, DefaultMessageListenerContainer> listenerMap;
     private DefaultMessageListenerContainer jmsListener;
+    private DefaultMessageListenerContainer allListener;
     private String conduitId;
     private AtomicLong messageCount;
-    private JmsTemplate jmsTemplate;
+    private JMSBusLifeCycleListener listener;
 
     public JMSConduit(EndpointInfo endpointInfo, EndpointReferenceType target, JMSConfiguration jmsConfig) {
         super(target);
         this.jmsConfig = jmsConfig;
         this.endpointInfo = endpointInfo;
         correlationMap = new ConcurrentHashMap<String, Exchange>();
-        listenerMap = new ConcurrentHashMap<String, DefaultMessageListenerContainer>();
-        
         conduitId = UUID.randomUUID().toString().replaceAll("-", "");
         messageCount = new AtomicLong(0);
-        listenerPool = new JMSListenerPool(new JMSListenerPoolableObjectFactory(jmsConfig,
-                                                                                this));
     }
     
     /**
@@ -105,100 +107,80 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
      * @return inMessage
      */
     public void sendExchange(final Exchange exchange, final Object request) {
-        DefaultMessageListenerContainer listener = null;
         LOG.log(Level.FINE, "JMSConduit send message");
 
         final Message outMessage = exchange.getOutMessage();
         if (outMessage == null) {
             throw new RuntimeException("Exchange to be sent has no outMessage");
         }
-
+        
+        boolean isTextPayload = JMSConstants.TEXT_MESSAGE_TYPE.equals(jmsConfig.getMessageType());
+        if (isTextPayload && MessageUtils.isTrue(outMessage.getContextualProperty(
+            org.apache.cxf.message.Message.MTOM_ENABLED)) 
+            && outMessage.getAttachments() != null && outMessage.getAttachments().size() > 0) {
+            org.apache.cxf.common.i18n.Message msg = 
+                new org.apache.cxf.common.i18n.Message("INVALID_MESSAGE_TYPE", LOG);
+            throw new ConfigurationException(msg);
+        }
+        
         final JMSMessageHeadersType headers = (JMSMessageHeadersType)outMessage
             .get(JMSConstants.JMS_CLIENT_REQUEST_HEADERS);
 
-        jmsTemplate = JMSFactory.createJmsTemplate(jmsConfig, headers);
-
-        javax.jms.Destination replyTo = null;
-        
+        final JmsTemplate jmsTemplate = JMSFactory.createJmsTemplate(jmsConfig, headers);
+        String userCID = headers != null ? headers.getJMSCorrelationID() : null;
+        DefaultMessageListenerContainer jmsList = jmsListener;
         if (!exchange.isOneWay()) {
-            if (jmsConfig.isUseMessageIDAsCorrelationID()) {
-                if (!exchange.isSynchronous()) {
-                    try {
-                        listener = (DefaultMessageListenerContainer)listenerPool.borrowObject();
-                    } catch (Exception ex) {
-                        LOG.log(Level.SEVERE, "Unable to borrow listener from pool: " + ex);
-                        throw new RuntimeException("Unable to borrow listener from pool.", ex);
-                    }
-                    replyTo = listener.getDestination();
-                } else {
-                    String replyToName = jmsConfig.getReplyDestination();
-                    replyTo = 
-                        JMSFactory.resolveOrCreateDestination(jmsTemplate, 
-                                                              replyToName, 
-                                                              jmsConfig.isPubSubDomain());
-                }
-            } else {
+            if (userCID == null || !jmsConfig.isUseConduitIdSelector()) { 
                 if (jmsListener == null) {
-                    jmsListener = JMSFactory.createJmsListener(jmsConfig, 
+                    jmsListener = JMSFactory.createJmsListener(jmsConfig, this, 
+                                                               jmsConfig.getReplyDestination(), 
+                                                               conduitId, 
+                                                               false);
+                    addBusListener(exchange.get(Bus.class));
+                }
+                jmsList = jmsListener;
+            } else {
+                if (allListener == null) {
+                    allListener = JMSFactory.createJmsListener(jmsConfig, 
                                                                this, 
                                                                jmsConfig.getReplyDestination(), 
-                                                               conduitId,
+                                                               null, 
                                                                true);
+                    addBusListener(exchange.get(Bus.class));
                 }
-                
-                replyTo = jmsListener.getDestination();
+                jmsList = allListener;
             }
-        }                       
+        }
+        
+        final javax.jms.Destination replyTo = exchange.isOneWay() ? null : jmsList.getDestination();
 
-        String correlationId = (headers != null && headers.isSetJMSCorrelationID()) 
+        final String correlationId = (headers != null && headers.isSetJMSCorrelationID()) 
             ? headers.getJMSCorrelationID() 
             : JMSUtils.createCorrelationId(jmsConfig.getConduitSelectorPrefix() + conduitId, 
                                            messageCount.incrementAndGet());
-        
-        if (jmsConfig.isUseMessageIDAsCorrelationID()) {
-            correlationId = null;
-        }
-                
-        if (exchange.isOneWay() && !jmsConfig.isEnforceSpec() && isSetReplyTo(outMessage)) {
-            String replyToName = (headers != null) ? headers.getJMSReplyTo() : null;
-            if (replyToName == null && jmsConfig.getReplyDestination() != null) {
-                replyToName = jmsConfig.getReplyDestination();
-            }
-            if (replyToName != null) {
-                replyTo = 
-                    JMSFactory.resolveOrCreateDestination(jmsTemplate, 
-                                                          replyToName, 
-                                                          jmsConfig.isPubSubDomain());
-            }
-        }
-        
-        String replyToDestination = jmsConfig.getReplyToDestination();
-        Destination destination = null;
-        
-        if (replyToDestination != null) {
-            destination = JMSFactory.resolveOrCreateDestination(jmsTemplate, 
-                                                                replyToDestination, 
-                                                                jmsConfig.isPubSubDomain());
-        }
-
-        final Destination rtd = destination == null ? replyTo : destination;
-        final String cid = correlationId;
-        class JMSConduitMessageCreator implements MessageCreator {
-            javax.jms.Message jmsMessage;
-                    
-            public JMSConduitMessageCreator() {
-            }
+            
+        MessageCreator messageCreator = new MessageCreator() {
             public javax.jms.Message createMessage(Session session) throws JMSException {
                 String messageType = jmsConfig.getMessageType();
-                jmsMessage = JMSUtils.buildJMSMessageFromCXFMessage(outMessage, request, messageType,
-                                                                    session, rtd,
-                                                                    cid);
+                final javax.jms.Message jmsMessage;
+                Destination replyToDestination = replyTo;
+                if (exchange.isOneWay() && !jmsConfig.isEnforceSpec()) {
+                    final String contextReplyToName = 
+                        (headers != null) ? headers.getJMSReplyTo() : null;
+                    if (contextReplyToName != null) {
+                        replyToDestination = 
+                            JMSFactory.resolveOrCreateDestination(jmsTemplate, 
+                                                                  contextReplyToName, 
+                                                                  jmsConfig.isPubSubDomain());
+                    }
+                }
+                jmsMessage = JMSUtils.buildJMSMessageFromCXFMessage(outMessage, request,
+                                                                    messageType, session, replyToDestination,
+                                                                    correlationId);
                 LOG.log(Level.FINE, "client sending request: ", jmsMessage);
                 return jmsMessage;
             }
-        }
-        
-        JMSConduitMessageCreator messageCreator = new JMSConduitMessageCreator();
+        };
 
         /**
          * If the message is not oneWay we will expect to receive a reply on the listener. To receive this
@@ -207,68 +189,68 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
          */
         if (!exchange.isOneWay()) {
             synchronized (exchange) {
-                if (correlationId != null) {
-                    correlationMap.put(correlationId, exchange);
-                }
+                correlationMap.put(correlationId, exchange);
                 jmsTemplate.send(jmsConfig.getTargetDestination(), messageCreator);
-                if (jmsConfig.isUseMessageIDAsCorrelationID()) {
+                
+                if (exchange.isSynchronous()) {
                     try {
-                        correlationId = messageCreator.jmsMessage.getJMSMessageID();                     
-                        handleMessageIDAsCorrelationID(correlationId,
-                                                       exchange,
-                                                       replyTo,
-                                                       listener);
-                    } catch (JMSException jmsex) {
-                        throw JmsUtils.convertJmsAccessException(jmsex);
-                    }
-                } else {
-                    if (exchange.isSynchronous()) {
-                        try {
-                            exchange.wait(jmsTemplate.getReceiveTimeout());
-                        } catch (InterruptedException e) {
-                            correlationMap.remove(correlationId);
-                            throw new RuntimeException(e);
-                        }
+                        exchange.wait(jmsTemplate.getReceiveTimeout());
+                    } catch (InterruptedException e) {
                         correlationMap.remove(correlationId);
-                        if (exchange.get(CORRELATED) == null) {
-                            throw new RuntimeException("Timeout receiving message with correlationId "
-                                                       + correlationId);
-                        }                                                        
-                    }                           
-                }                
+                        throw new RuntimeException(e);
+                    }
+                    correlationMap.remove(correlationId);
+                    if (exchange.get(CORRELATED) == null) {
+                        throw new RuntimeException("Timeout receiving message with correlationId "
+                                                   + correlationId);
+                    }
+                }
             }
         } else {
             jmsTemplate.send(jmsConfig.getTargetDestination(), messageCreator);
         }
     }
 
-    private void handleMessageIDAsCorrelationID(String correlationId,
-                        Exchange exchange,
-                        Destination replyTo,
-                        DefaultMessageListenerContainer listener) {
-        String messageSelector = "JMSCorrelationID='" + correlationId + "'";
-        if (exchange.isSynchronous()) {
-            javax.jms.Message message = 
-                jmsTemplate.receiveSelected(replyTo, messageSelector);
-            if (message != null) {
-                handleMessage(exchange, message);
-            } else {
-                throw new RuntimeException("Timeout receiving message with correlationId "
-                                           + correlationId);
-            }
-        } else {
-            listener.setMessageListener(this);
-            correlationMap.put(correlationId, exchange);
-            listener.setMessageSelector("JMSCorrelationID='" + correlationId + "'");
+    static class JMSBusLifeCycleListener implements BusLifeCycleListener {
+        final WeakReference<JMSConduit> ref;
+        BusLifeCycleManager blcm;
+        JMSBusLifeCycleListener(JMSConduit c, BusLifeCycleManager b) {
+            ref = new WeakReference<JMSConduit>(c);
+            blcm = b;
+            blcm.registerLifeCycleListener(this);
+        }
+        
+        public void initComplete() {
+        }
 
-            if (!listener.isActive()) {
-                listener.initialize();
+        public void postShutdown() {
+        }
+
+        public void preShutdown() {
+            unreg();
+            blcm = null;
+            JMSConduit c = ref.get();
+            if (c != null) {
+                c.listener = null;
+                c.close();
+            }
+        }
+        public void unreg() {
+            if (blcm != null) {
+                blcm.unregisterLifeCycleListener(this);
             }
-            
-            listenerMap.put(correlationId, listener);
         }
     }
-    
+    private synchronized void addBusListener(Bus bus) {
+        if (listener == null && bus != null) {
+            BusLifeCycleManager blcm = bus.getExtension(BusLifeCycleManager.class);
+            if (blcm != null) {
+                listener = new JMSBusLifeCycleListener(this,
+                                                       blcm);
+            }
+        }
+    }
+
     /**
      * When a message is received on the reply destination the correlation map is searched for the
      * correlationId. If it is found the message is converted to a CXF message and the thread sending the
@@ -283,26 +265,10 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
         }
 
         Exchange exchange = correlationMap.remove(correlationId);
-        
-        DefaultMessageListenerContainer listener = listenerMap.get(correlationId);
-
-        if (listener != null) {
-            try {
-                listenerPool.returnObject(listener);
-            } catch (Exception ex) {
-                LOG.log(Level.WARNING, "Could not return listener to pool: " + ex);
-            }
-        }
-        
         if (exchange == null) {
             LOG.log(Level.WARNING, "Could not correlate message with correlationId " + correlationId);
             return;
         }
-        
-        handleMessage(exchange, jmsMessage);
-    }
-    
-    private void handleMessage(Exchange exchange, javax.jms.Message jmsMessage) {
         Message inMessage = new MessageImpl();
         exchange.setInMessage(inMessage);
         LOG.log(Level.FINE, "client received reply: ", jmsMessage);
@@ -330,38 +296,19 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
     }
 
     public void close() {
-        synchronized (this) {
-            if (listenerPool != null) {
-                try {
-                    listenerPool.close();
-                } catch (Exception ex) {
-                    LOG.log(Level.WARNING, "Failed to close JMSListener pool: " + ex);
-                } finally {
-                    listenerPool = null;
-                }
-            }
-            
-            if (jmsListener != null) {
-                jmsListener.shutdown();
-                jmsListener = null;
-            }
-            
-            if (jmsTemplate != null) {
-                Object obj = jmsTemplate.getConnectionFactory();
-                if (obj instanceof SingleConnectionFactory) {
-                    //Since all listeners/templates use same config/connectionfactory, it is only
-                    //necessary to call destroy() once.
-                    LOG.log(Level.FINE, "Destroying SingleConnectionFactory from template .....");
-                    ((SingleConnectionFactory)obj).destroy();
-                }
-                
-                jmsTemplate = null;
-            }        
-            
-            LOG.log(Level.FINE, "JMSConduit closed ");
+        if (listener != null) {
+            listener.unreg();
+            listener = null;
         }
+        if (jmsListener != null) {
+            jmsListener.shutdown();
+        }
+        if (allListener != null) {
+            allListener.shutdown();
+        }
+        LOG.log(Level.FINE, "JMSConduit closed ");
     }
-    
+
     protected Logger getLogger() {
         return LOG;
     }
@@ -374,14 +321,19 @@ public class JMSConduit extends AbstractConduit implements JMSExchangeSender, Me
         this.jmsConfig = jmsConfig;
     }
 
-    protected static boolean isSetReplyTo(Message message) {         
-        Boolean ret = (Boolean)message.get(JMSConstants.JMS_SET_REPLY_TO);
-        return ret == null || (ret != null && ret.booleanValue());
-    }
-
     @Override
     protected void finalize() throws Throwable {
-        close();
+        if (listener != null) {
+            listener.unreg();
+            listener = null;
+        }
+        if (jmsListener != null) {
+            jmsListener.shutdown();
+        }
+        if (allListener != null) {
+            allListener.shutdown();
+        }
         super.finalize();
-    }   
+    }
+
 }
