@@ -20,14 +20,18 @@
 package org.apache.cxf.systest.jaxrs;
 
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
 import java.net.URL;
 import java.util.Calendar;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 
-import javax.ws.rs.ConsumeMime;
+import javax.ws.rs.Consumes;
 import javax.ws.rs.DELETE;
 import javax.ws.rs.DefaultValue;
 import javax.ws.rs.GET;
@@ -37,12 +41,18 @@ import javax.ws.rs.POST;
 import javax.ws.rs.PUT;
 import javax.ws.rs.Path;
 import javax.ws.rs.PathParam;
-import javax.ws.rs.ProduceMime;
+import javax.ws.rs.Produces;
 import javax.ws.rs.QueryParam;
 import javax.ws.rs.WebApplicationException;
+import javax.ws.rs.core.Context;
+import javax.ws.rs.core.GenericEntity;
+import javax.ws.rs.core.HttpHeaders;
+import javax.ws.rs.core.MediaType;
 import javax.ws.rs.core.MultivaluedMap;
 import javax.ws.rs.core.PathSegment;
 import javax.ws.rs.core.Response;
+import javax.ws.rs.core.SecurityContext;
+import javax.ws.rs.core.StreamingOutput;
 import javax.xml.bind.JAXBElement;
 import javax.xml.bind.annotation.adapters.XmlAdapter;
 import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
@@ -62,7 +72,11 @@ public class BookStore {
     private String currentBookId;
     @PathParam("CDId")
     private String currentCdId;
-
+    @Context
+    private HttpHeaders httpHeaders;
+    @Context 
+    private SecurityContext securityContext;
+    
     public BookStore() {
         init();
     }
@@ -75,9 +89,16 @@ public class BookStore {
         throw new WebApplicationException(response);
     }
     
+    @GET
+    @Path("propogateexception")
+    public Book propogateException() throws BookNotFoundFault {
+        throw new BookNotFoundFault("Book Exception");
+    }
+    
+    
     @GET
     @Path("books/check/{id}")
-    @ProduceMime("text/plain")
+    @Produces("text/plain")
     public boolean checkBook(@PathParam("id") Long id) {
         return books.containsKey(id);
     }
@@ -103,7 +124,7 @@ public class BookStore {
     
     @POST
     @Path("/unsupportedcontenttype")
-    @ConsumeMime("application/xml")
+    @Consumes("application/xml")
     public String unsupportedContentType() {
         throw new IllegalStateException("This op is not expected to be invoked");
     }
@@ -114,7 +135,7 @@ public class BookStore {
         String url2 = new URL(urlValue).toString();
         int index = url2.lastIndexOf('/');
         return doGetBook(url2.substring(index + 1));
-    } 
+    }
     
     @GET
     @Path("/segment/{pathsegment}/")
@@ -148,10 +169,30 @@ public class BookStore {
     @GET
     @Path("/bookheaders/")
     public Book getBookByHeader(@HeaderParam("BOOK") List<String> ids) throws Exception {
+        List<MediaType> types = httpHeaders.getAcceptableMediaTypes();
+        if (types.size() != 2 
+            || !"text/xml".equals(types.get(0).toString())
+            || !MediaType.APPLICATION_XML_TYPE.isCompatible(types.get(1))) {
+            throw new WebApplicationException();
+        }
+        List<Locale> locales = httpHeaders.getAcceptableLanguages();
+        if (locales.size() != 2 
+            || !"en".equals(locales.get(0).getLanguage())
+            || !"da".equals(locales.get(1).getLanguage())) {
+            throw new WebApplicationException();
+        }
         
         return doGetBook(ids.get(0) + ids.get(1) + ids.get(2));
     }
     
+    @GET
+    @Path("/bookheaders2/")
+    public Book getBookByHeader(@DefaultValue("123") @HeaderParam("BOOK2") String id) 
+        throws Exception {
+        return doGetBook(id);
+    }
+    
+    
     @GET
     @Path("/bookquery")
     public Book getBookByURLQuery(@QueryParam("urlid") String urlValue) throws Exception {
@@ -159,13 +200,66 @@ public class BookStore {
         int index = url2.lastIndexOf('/');
         return doGetBook(url2.substring(index + 1));
     }
+    
+    @GET
+    @Path("/bookidarray")
+    public Book getBookByURLQuery(@QueryParam("id") String[] ids) throws Exception {
+        if (ids == null || ids.length != 3) {
+            throw new WebApplicationException(); 
+        }
+        return doGetBook(ids[0] + ids[1] + ids[2]);
+    }
 
+    @GET
+    @Path("/securebooks/{bookId}/")
+    @Produces("application/xml")
+    public Book getSecureBook(@PathParam("bookId") String id) throws BookNotFoundFault {
+        if (!securityContext.isSecure()) {
+            throw new WebApplicationException(Response.status(403).entity("Unsecure link").build());
+        }
+        return doGetBook(id);
+    }
+    
+    @GET
+    @Path("/genericbooks/{bookId}/")
+    @Produces("application/xml")
+    public GenericEntity<GenericHandler<Book>> getGenericBook(@PathParam("bookId") String id) 
+        throws BookNotFoundFault {
+        return new GenericEntity<GenericHandler<Book>>(new GenericHandler<Book>(doGetBook(id))) { };
+    }
+    
+    @GET
+    @Path("/genericresponse/{bookId}/")
+    @Produces("application/xml")
+    public Response getGenericResponseBook(@PathParam("bookId") String id) 
+        throws BookNotFoundFault {
+        return Response.ok(getGenericBook(id)).build();
+    }
+    
     @GET
     @Path("/books/{bookId}/")
+    @Produces("application/xml")
     public Book getBook(@PathParam("bookId") String id) throws BookNotFoundFault {
         return doGetBook(id);
     }
     
+    @GET
+    @Path("/books/wrapper/{bookId}/")
+    @Produces("application/xml")
+    public BookWrapper getWrappedBook(@PathParam("bookId") Long id) throws BookNotFoundFault {
+        BookWrapper bw = new BookWrapper();
+        Book b = new Book("CXF in Action", 99999L);
+        bw.setBook(b);
+        return bw;
+    }
+    
+    @GET
+    @Path("/books/wrapper2/{bookId}/")
+    @Produces("application/xml")
+    public Book getWrappedBook2(@PathParam("bookId") Long id) throws BookNotFoundFault {
+        return new Book("CXF in Action", 99999L);
+    }
+    
     @GET
     @Path("books/custom/{bookId:\\d\\d\\d}")
     public Book getBookCustom(@PathParam("bookId") String id) throws BookNotFoundFault {
@@ -217,13 +311,34 @@ public class BookStore {
     
     @GET
     @Path("/books/{bookId}/")
-    @ProduceMime("application/json")
+    @Produces("application/json;q=0.9")
     public Book getBookAsJSON() throws BookNotFoundFault {
         return doGetBook(currentBookId);
     }
     
+    
+    @GET
+    @Path("/books/buffer")
+    @Produces("application/bar")
+    public InputStream getBufferedBook() {
+        return getClass().getResourceAsStream("resources/expected_get_book123.txt");
+    }
+    
+    @GET
+    @Path("/books/fail-early")
+    @Produces("application/bar")
+    public StreamingOutput failEarlyInWrite() {
+        return new StreamingOutputImpl(true);
+    }
+    
+    @GET
+    @Path("/books/fail-late")
+    @Produces("application/bar")
+    public StreamingOutput writeToStreamAndFail() {
+        return new StreamingOutputImpl(false);
+    }
+    
     private Book doGetBook(String id) throws BookNotFoundFault {
-        //System.out.println("----invoking getBook with id: " + id);
         Book book = books.get(Long.parseLong(id));
         if (book != null) {
             return book;
@@ -253,7 +368,7 @@ public class BookStore {
     
     @GET
     @Path("/booknames/{bookId}/")
-    @ProduceMime("text/*")
+    @Produces("text/*")
     public String getBookName(@PathParam("bookId") int id) throws BookNotFoundFault {
         Book book = books.get(new Long(id));
         if (book != null) {
@@ -267,19 +382,35 @@ public class BookStore {
 
     @POST
     @Path("/books")
-    @ProduceMime("text/xml")
-    @ConsumeMime("application/xml")
+    @Produces("text/xml")
+    @Consumes("application/xml")
     public Response addBook(Book book) {
-        book.setId(++bookId);
+        String ct1 = httpHeaders.getMediaType().toString();
+        String ct2 = httpHeaders.getRequestHeader("Content-Type").get(0);
+        String ct3 = httpHeaders.getRequestHeaders().getFirst("Content-Type");
+        if (!("application/xml".equals(ct1) && ct1.equals(ct2) && ct1.equals(ct3))) {
+            throw new RuntimeException("Unexpected content type");
+        }
+        
+        book.setId(bookId + 1);
         books.put(book.getId(), book);
 
         return Response.ok(book).build();
     }
+    
+    @POST
+    @Path("/booksinfo")
+    @Produces("text/xml")
+    @Consumes("application/xml")
+    public Response addBook(@XmlJavaTypeAdapter(BookInfoAdapter.class) 
+                            BookInfo bookInfo) {
+        return Response.ok(bookInfo.asBook()).build();
+    }
 
     @POST
     @Path("/binarybooks")
-    @ProduceMime("text/xml")
-    @ConsumeMime("application/octet-stream")
+    @Produces("text/xml")
+    @Consumes("application/octet-stream")
     public Response addBinaryBook(long[] book) {
         return Response.ok(book).build();
     }
@@ -300,6 +431,24 @@ public class BookStore {
         return r;
     }
     
+    @PUT
+    @Path("/books/{id}")
+    public Response createBook(@PathParam("id") Long id) {
+        Book b = books.get(id);
+
+        Response r;
+        if (b == null) {
+            Book newBook = new Book();
+            newBook.setId(id);
+            books.put(newBook.getId(), newBook);
+            r = Response.ok().build();
+        } else {
+            r = Response.notModified().build();
+        }
+
+        return r;
+    }
+    
     @PUT
     @Path("/bookswithdom/")
     public DOMSource updateBook(DOMSource ds) {
@@ -309,7 +458,7 @@ public class BookStore {
     
     @PUT
     @Path("/bookswithjson/")
-    @ConsumeMime("application/json")
+    @Consumes("application/json")
     public Response updateBookJSON(Book book) {
         Book b = books.get(book.getId());
 
@@ -359,8 +508,8 @@ public class BookStore {
 
     @POST
     @Path("/booksplain")
-    @ConsumeMime("text/plain")
-    @ProduceMime("text/plain")
+    @Consumes("text/plain")
+    @Produces("text/plain")
     public Long echoBookId(long theBookId) {
         return new Long(theBookId);
     }
@@ -375,7 +524,7 @@ public class BookStore {
 
     @GET
     @Path("/cdwithmultitypes/{CDId}/")
-    @ProduceMime({"application/xml", "application/json" }) 
+    @Produces({"application/xml", "application/bar+xml", "application/json" }) 
     public CD getCDWithMultiContentTypes(@PathParam("CDId") String id) {
         CD cd = cds.get(Long.parseLong(id));
 
@@ -411,13 +560,20 @@ public class BookStore {
         cds.put(cd1.getId(), cd1);
     }
     
-    private static class BookInfo {
+    static class BookInfo {
         private String name;
         private long id;
         
+        public BookInfo() {
+            
+        }
+        
         public BookInfo(Book b) {
             this.name = b.getName();
             this.id = b.getId();
+            if (id == 0) {
+                id = 124;
+            }
         }
         
         public String getName() {
@@ -427,6 +583,13 @@ public class BookStore {
         public long getId() {
             return id;
         }
+       
+        public Book asBook() {
+            Book b = new Book();
+            b.setId(id);
+            b.setName(name);
+            return b;
+        }
     }
     
     public static class BookInfoAdapter extends XmlAdapter<Book, BookInfo> {
@@ -437,18 +600,38 @@ public class BookStore {
         }
 
         @Override
-        public BookInfo unmarshal(Book v) throws Exception {
-            // TODO Auto-generated method stub
-            return null;
+        public BookInfo unmarshal(Book b) throws Exception {
+            return new BookInfo(b);
         }
         
     }
     
-    private static class BadBook {
+    static class BadBook {
         public BadBook(String s) {
             throw new RuntimeException("The bad book");
         }
     }
+    
+    private static class StreamingOutputImpl implements StreamingOutput {
+
+        private boolean failEarly;
+        
+        public StreamingOutputImpl(boolean failEarly) {
+            this.failEarly = failEarly;
+        }
+        
+        public void write(OutputStream output) throws IOException, WebApplicationException {
+            if (failEarly) {
+                throw new WebApplicationException(
+                     Response.status(410).type("text/plain")
+                     .entity("This is supposed to go on the wire").build());
+            } else {
+                output.write("This is not supposed to go on the wire".getBytes());
+                throw new WebApplicationException(410);
+            }
+        } 
+        
+    }
 }
 
 
