@@ -275,8 +275,16 @@ public class ClientServerMiscTest extends AbstractBusClientServerTestBase {
         assertEquals("Hello", echoMsg);
     }
     private void runDocLitTest(DocLitWrappedCodeFirstService port) throws Exception {
+        long start = System.currentTimeMillis();
+        port.doOneWay();
+        assertTrue((System.currentTimeMillis() - start) < 500);
+        
+        assertEquals("Hello", port.echoStringNotReallyAsync("Hello"));
+        
         Set<Foo> fooSet = port.getFooSet();
         assertEquals(2, fooSet.size());
+        assertEquals("size: 2", port.doFooList(new ArrayList<Foo>(fooSet)));
+        
         
         assertEquals(24, port.echoIntDifferentWrapperName(24));
         
