@@ -28,9 +28,14 @@ import java.util.HashMap;
 import java.util.Map;
 import java.util.UUID;
 
+import javax.activation.DataHandler;
 import javax.xml.namespace.QName;
+import javax.xml.ws.Binding;
 import javax.xml.ws.BindingProvider;
 import javax.xml.ws.Endpoint;
+import javax.xml.ws.Holder;
+import javax.xml.ws.soap.SOAPBinding;
+
 
 import org.apache.cxf.endpoint.Client;
 import org.apache.cxf.frontend.ClientProxy;
@@ -48,6 +53,8 @@ import org.apache.cxf.hello_world_jms.HelloWorldServiceAppCorrelationIDStaticPre
 import org.apache.cxf.hello_world_jms.HelloWorldServiceRuntimeCorrelationIDDynamicPrefix;
 import org.apache.cxf.hello_world_jms.HelloWorldServiceRuntimeCorrelationIDStaticPrefix;
 import org.apache.cxf.hello_world_jms.NoSuchCodeLitFault;
+import org.apache.cxf.jms_mtom.JMSMTOMPortType;
+import org.apache.cxf.jms_mtom.JMSMTOMService;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
 import org.apache.cxf.transport.jms.AddressType;
@@ -59,21 +66,15 @@ import org.apache.cxf.transport.jms.JMSPropertyType;
 import org.apache.hello_world_doc_lit.Greeter;
 import org.apache.hello_world_doc_lit.PingMeFault;
 import org.apache.hello_world_doc_lit.SOAPService2;
-import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Test;
 
-import org.springframework.context.ApplicationContext;
 import org.springframework.context.support.ClassPathXmlApplicationContext;
 
 public class JMSClientServerTest extends AbstractBusClientServerTestBase {
     
-    protected static boolean serversStarted;
-
-    @Before
-    public void startServers() throws Exception {
-        if (serversStarted) {
-            return;
-        }
+    @BeforeClass
+    public static void startServers() throws Exception {
         Map<String, String> props = new HashMap<String, String>();                
         if (System.getProperty("activemq.store.dir") != null) {
             props.put("activemq.store.dir", System.getProperty("activemq.store.dir"));
@@ -86,7 +87,6 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
 
         assertTrue("server did not launch correctly", 
                    launchServer(Server.class, false));
-        serversStarted = true;
     }
     
     public URL getWSDLURL(String s) throws Exception {
@@ -195,23 +195,6 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
             throw (Exception)ex.getCause();
         }
     }
-    
-    @Test
-    public void testJmsClientWithHandler() throws Exception {
-        
-        ApplicationContext context = 
-            new ClassPathXmlApplicationContext("org/apache/cxf/systest/jms/JMSClientWithHandler.xml");
-        Greeter greeter = (Greeter) context.getBean("myClient");
-        try {
-            greeter.sayHi();
-            fail("Expect exception here");
-        } catch (Exception ex) {
-            assertTrue("Get a wrong exception", ex instanceof RuntimeException);
-            assertEquals("Get a wrong exception", "VRS Exception", ex.getMessage());
-        }
-        
-    }
-    
 
     @Test
     public void testBasicConnection() throws Exception {
@@ -258,7 +241,6 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
         } catch (Exception t) {
             throw t;
         }
-
     }
     
     @Test
@@ -308,7 +290,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
                 greeter.greetMeOneWay("JMS:PubSub:Milestone-" + idx);
             }
             //Give some time to complete one-way calls.
-            Thread.sleep(100L);
+            Thread.sleep(50L);
         } catch (UndeclaredThrowableException ex) {
             throw (Exception)ex.getCause();
         }
@@ -332,7 +314,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
                 greeter.greetMeOneWay("JMS:PubSub:Milestone-" + idx);
             }
             //Give some time to complete one-way calls.
-            Thread.sleep(100L);
+            Thread.sleep(50L);
         } catch (UndeclaredThrowableException ex) {
             throw (Exception)ex.getCause();
         }
@@ -482,7 +464,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
             }
         }
     }
-
+    
     @Test
     public void testQueueOneWaySpecCompliantConnection() throws Exception {
         QName serviceName = getServiceName(new QName("http://cxf.apache.org/hello_world_jms", 
@@ -685,7 +667,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
         
         HelloWorldPortType port = service.getPort(portName, HelloWorldPortType.class);
         
-        for (int i = 0; i < 100; ++i) {
+        for (int i = 0; i < 10; ++i) {
             ClientRunnable client =  
                 new ClientRunnable(port,
                     new CorrelationIDFactory() {
@@ -767,7 +749,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
         HelloWorldPortType portEng = service.getPort(portNameEng, HelloWorldPortType.class);
         HelloWorldPortType portSales = service.getPort(portNameSales, HelloWorldPortType.class);
         
-        for (int i = 0; i < 100; ++i) {
+        for (int i = 0; i < 10; ++i) {
             ClientRunnable client =  new ClientRunnable(portEng, "com.mycompany.eng:");
             Thread thread = new Thread(client);
             threads.add(thread);
@@ -812,7 +794,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
         
         HelloWorldPortType port = service.getPort(portName, HelloWorldPortType.class);
         
-        for (int i = 0; i < 100; ++i) {
+        for (int i = 0; i < 10; ++i) {
             ClientRunnable client =  
                 new ClientRunnable(port);
             
@@ -853,8 +835,7 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
             BindingProvider  bp = null;
             
             if (handler instanceof BindingProvider) {
-                bp = (BindingProvider)handler;
-                //System.out.println(bp.toString());
+                bp = (BindingProvider)handler;                
                 Map<String, Object> requestContext = bp.getRequestContext();
                 JMSMessageHeadersType requestHeader = new JMSMessageHeadersType();
                 requestHeader.setJMSCorrelationID("JMS_SAMPLE_CORRELATION_ID");
@@ -901,4 +882,28 @@ public class JMSClientServerTest extends AbstractBusClientServerTestBase {
         }
     }
     
+    @Test
+    public void testMTOM() throws Exception {
+        QName serviceName = new QName("http://cxf.apache.org/jms_mtom", "JMSMTOMService");
+        QName portName = new QName("http://cxf.apache.org/jms_mtom", "MTOMPort");
+
+        URL wsdl = getClass().getResource("/wsdl/jms_test_mtom.wsdl");
+        assertNotNull(wsdl);
+
+        JMSMTOMService service = new JMSMTOMService(wsdl, serviceName);
+        assertNotNull(service);
+
+        JMSMTOMPortType mtom = service.getPort(portName, JMSMTOMPortType.class);
+        Binding binding = ((BindingProvider)mtom).getBinding();
+        ((SOAPBinding)binding).setMTOMEnabled(true);
+
+        Holder<String> name = new Holder<String>("Sam");
+        URL fileURL = this.getClass().getResource("/org/apache/cxf/systest/jms/JMSClientServerTest.class");
+        Holder<DataHandler> handler1 = new Holder<DataHandler>();
+        handler1.value = new DataHandler(fileURL);
+        int size = handler1.value.getInputStream().available();
+        mtom.testDataHandler(name, handler1);
+        int size2 = handler1.value.getInputStream().available();
+        assertTrue("The response file is not same with the sent file.", size == size2);
+    }
 }
