@@ -59,6 +59,11 @@ public abstract class AbstractTypeTestClient
     }
 
     public boolean shouldRunTest(String name) {
+        if (System.getProperty("java.vendor").contains("IBM")
+            && "GMonth".equals(name)) {
+            //the validator in ibm doesn't like this type.
+            return false;
+        }
         return true;
     }
     
@@ -277,7 +282,7 @@ public abstract class AbstractTypeTestClient
         if (!shouldRunTest("UnsignedInt")) {
             return;
         }
-        long valueSets[][] = {{11, 20}, {1, 0}, {0, ((long)Integer.MAX_VALUE) * 2 + 1}};
+        long valueSets[][] = {{0, ((long)Integer.MAX_VALUE) * 2 + 1}, {11, 20}, {1, 0}};
 
         for (int i = 0; i < valueSets.length; i++) {
             long x = valueSets[i][0];
@@ -294,7 +299,8 @@ public abstract class AbstractTypeTestClient
                 ret = rpcClient.testUnsignedInt(x, y, z);
             }
             if (!perfTestOnly) {
-                assertEquals("testUnsignedInt(): Incorrect value for inout param", Long.valueOf(x), y.value);
+                assertEquals("testUnsignedInt(): Incorrect value for inout param",
+                             Long.valueOf(x), y.value);
                 assertEquals("testUnsignedInt(): Incorrect value for out param",
                              Long.valueOf(yOrig), z.value);
                 assertEquals("testUnsignedInt(): Incorrect return value", x, ret);
@@ -538,8 +544,11 @@ public abstract class AbstractTypeTestClient
             buffer2.append((char)('A' + (x % 26)));
         }
 
-        String valueSets[][] = {{"hello", "world"}, {"is pi > 3 ?", " is pi < 4\\\""},
-                                {"<illegal_tag/>", ""}, {buffer.toString(), buffer2.toString()}};
+        String valueSets[][] = {{"hello", "world"},
+                                {"is pi > 3 ?", " is pi < 4\\\""},
+                                {"<illegal_tag/>", ""},
+                                {buffer.toString(), buffer2.toString()},
+                                {"jon&marry", "marry&john"}};
 
         for (int i = 0; i < valueSets.length; i++) {
             String x = valueSets[i][0];
